# Lab 3
# command file for analyzing hue naming data
# Lewis O. Harvey, Jr.
# Department of Psychology
# University of Colorado

# read file into data frame
#fn <- "SampleData.txt"
fn <- file.choose()
df <- read.delim(fn, header=TRUE)

# Compute opponent processs and add columns to data frame
df <- cbind(df, red_green = df$red - df$green)
df <- cbind(df, yellow_blue = df$yellow - df$blue)
df <- cbind(df, total = df$red + df$yellow + df$green + df$blue)

# Compute the mean wavelengths of the pure colors
pure.blue <- mean(df$pure_blue, na.rm = TRUE)
pure.green <- mean(df$pure_green, na.rm = TRUE)
pure.yellow <- mean(df$pure_yellow, na.rm = TRUE)

par(ask = TRUE)
par(mfrow=c(2,1))	# two graphs per figure

# ------------------------------------------------------
# plot the individual color percentages
# ------------------------------------------------------

# red and green percentages
plot(red ~ wavelength, data = df,
	col = "red", type = "b", pch = 19,
	xlim = c(400, 700),
	ylim = c(0, 100),
	xlab = "Wavelength in nm",
	ylab = "Percentage of Color",
	main = "Fig 1: Red and Green Responses")
points(green ~ wavelength, data = df,
	col = "green", type = "b", pch = 19,
	xlim = c(400, 700),
	ylim = c(-100, 100))

# yellow and blue percentages
plot(yellow ~ wavelength, data = df,
	col = "orange", type = "b", pch = 19,
	xlim = c(400, 700),
	ylim = c(0, 100),
	xlab = "Wavelength in nm",
	ylab = "Percentage of Color",
	main = "Fig 1: Yellow and Blue Responses")
points(blue ~ wavelength, data = df,
	col = "blue", type = "b", pch = 19,
	xlim = c(400, 700),
	ylim = c(-100, 100))

	
# ------------------------------------------------------
# plot the opponent processes
# ------------------------------------------------------

# plot the red-green process
df.red <- subset(df, red_green > 0)
df.green <- subset(df, red_green <= 0)
plot(red_green ~ wavelength, data = df.red,
	type = "p", col = "red", pch = 19,
	xlim = c(400, 700),
	ylim = c(-100, 100),
	xlab = "Wavelength in nm",
	ylab = "Red - Green Response",
	main = "Fig 2: Red - Green Chromatic Mechanism")
points(red_green ~ wavelength, data = df.green,
	type = "p", col = "green", pch = 19)
abline(h = 0)

# fit a polynomial to the opponent process data and
# plot the best-fitting line
rg.poly <- lm(red_green ~ poly(wavelength, degree = 8), data = df)
rg.coef <- coef(rg.poly)
rg.root <- polyroot(rg.coef)
x <- 400:700
lines(x, predict(rg.poly, data.frame(wavelength = x)))
abline(v = pure.blue, col = "blue")
abline(v = pure.yellow, col = "orange")

# plot the yellow-blue opponent process
df.yellow <- subset(df, yellow_blue > 0)
df.blue <- subset(df, yellow_blue <= 0)
plot(yellow_blue ~ wavelength, data = df.yellow,
	type = "p", col = "orange", pch = 19,
	xlim = c(400, 700),
	ylim = c(-100, 100),
	xlab = "Wavelength in nm",
	ylab = "Yellow - Blue Response",
	main = "Fig 2: Yellow - Blue Chromatic Mechanism")
points(yellow_blue ~ wavelength, data = df.blue,
	type = "p", col = "blue", pch = 19)
abline(h = 0)

# fit a polynomial to the opponent process data and
# plot the best-fitting line
yb.poly <- lm(yellow_blue ~ poly(wavelength, degree = 8), data = df)
yb.coef <- coef(yb.poly)
yb.root <- polyroot(yb.coef)
x <- 400:700
lines(x, predict(yb.poly, data.frame(wavelength = x)))
abline(v = pure.green, col = "green")

# plot total percentage as a function of wavelength
plot(total ~ wavelength, data = df, pch = 19,
	ylim = c(0, 100),
	xlab = "Wavelength in nm",
	ylab = "Total Percentage",
	main = "Sum of Red, Yellow, Green, and Blue Percentages")
# compute a linear regression on total
tot.lm <- lm(total ~ wavelength, data = df)
lines(tot.lm$fitted ~ df$wavelength)

# plot the residual from the total ~ wavelength regression
plot(tot.lm$residual ~ df$wavelength, pch = 19,
	ylim = c(-50, 50),
	xlab = "Wavelength in nm",
	ylab = "Residuals")
abline(h=0)


par(mfrow=c(1,1))
